/**
 * Default controller
 */

module.exports = {
  // default crud actions
  find: function(req, res, next) {
    res.locals.Model.findAndCountAll(res.locals.query)
    .done(function(err, record) {
      if (err) return res.serverError(err);

      res.locals.metadata.count = record.count;
      res.locals.record = record.rows;

      return res.ok();
    });
  },

  create: function(req, res) {
    var we = req.getWe();

    res.locals.Model.create(req.body)
    .done(function(err, record) {
      if (err) return res.send(err);
      // generate a unique url
      we.url.generate(req, res, record, function(err, url) {
        if (err) {
          we.log.error(err);
          res.addMessage('error', err);
        }

        res.locals.metadata.url = url;
        return res.created(record);
      })
    });
  },
  findOne: function(req, res, next) {
    var we = req.getWe();
    var id = req.params.id;

    res.locals.Model.find({
      where: { id: id},
      include: [{ all: true,  attributes: ['id'] }]
    })
    .done(function(err, record) {
      if (err) return res.serverError(err);
      if (!record) return res.notFound();

      res.locals.record = record;

      we.hooks.trigger('we:after:send:ok:response', {
        res: res,
        req: req
      }, function() {
        return res.ok();
      });
    });
  },
  update: function(req, res, next) {
    var id = req.params.id;

    res.locals.Model.find(id)
    .done(function(err, record) {
      if (err) return res.serverError(err);
      if (!record) return res.notFound();

      record.updateAttributes(req.body)
      .done(function(err) {
        if (err) return res.serverError(err);

        return res.ok(record);
      });
    });
  },
  destroy: function(req, res, next) {
    var id = req.params.id;

    res.locals.Model.find(id)
    .done(function(err, record) {
      if (err) return res.serverError(err);
      if (!record) return res.notFound();

      record.destroy(req.body)
      .done(function(err) {
        if (err) return res.serverError(err);
        return res.status(204).send();
      });
    });
  },
  // pages actions
  editPage: function(req, res, next) {
    console.log('TODO');
  },
  // atribute actions
  getAttribute: function(req, res, next) {
    console.log('TODO');
  },
  updateAttribute: function(req, res, next) {
    console.log('TODO');
  },
  deleteAttribute: function(req, res, next) {
    console.log('TODO');
  },
  // association actions
  addRecord: function(req, res, next) {
    console.log('TODO');
  },
  removeRecord: function(req, res, next) {
    console.log('TODO');
  },
  getRecord: function(req, res, next) {
    console.log('TODO');
  }
}

function parseValues(req) {
  var we =  req.getWe();

  var modelAtributes = Object.keys( we.modelsConfigs[res.locals.model].definition );
}